# Univerxel <!-- omit in toc -->

Work in progress galaxy down to atom (mostly centimeter) online voxel game

## Table of Contents <!-- omit in toc -->

- [About The Project](#about-the-project)
  - [Built With](#built-with)
- [Run it](#run-it)
  - [Prerequisites](#prerequisites)
  - [Usage](#usage)
- [Build](#build)
  - [Prerequisites](#prerequisites-1)
    - [Optionally](#optionally)
  - [Installation](#installation)
    - [Additionally](#additionally)
- [RoadMap](#roadmap)
- [License](#license)
- [Contact](#contact)


<!-- ABOUT THE PROJECT -->
## About The Project

Experimental project using OpenGL and Vulkan.


### Built With

* C++
* OpenGL
* Vulkan
* ImGui
* PicoQUIC
* FasNoiseSIMD
* Love and insomnia


<!-- GETTING STARTED -->
## Run it

Get a release compatible with your system window library and processor capabilities, or [build](#build) it

### Prerequisites

* OpenGL or Vulkan driver
* OpenSSL 1.1+

### Usage

```sh
./univerxel
```

Edit `config.toml`
<!-- MAYBE: config description -->

Profit !

## Build

To get a local copy up and running, follow these simple steps.

### Prerequisites

* C++17
* CMake 3.11
* System window headers *(xorg-dev)*
* OpenGL, Vulkan and OpenSSL headers

#### Optionally

* Python: utility scripts
* [Tracy](https://github.com/wolfpld/tracy) v0.7: profiling
* [glslc](https://github.com/KhronosGroup/glslang): build vk shaders
* Imagemagick: build textures (convert jpg to png)
* [Compressonator](https://github.com/GPUOpen-Tools/compressonator): build textures (compress dds)
* [Doxygen](https://www.doxygen.nl/index.html): API docs
* [Sphinx](https://www.sphinx-doc.org/) [rtd_theme](https://github.com/readthedocs/sphinx_rtd_theme) [Breathe](https://breathe.readthedocs.io/en/latest/): sphinx pages

### Installation

1. Clone the project repo
```sh
git lfs clone --recursive https://framagit.org/univerxel/univerxel.git
```
2. Create build folder and move
```sh
mkdir build && cd build
```
3. Build CMake
```sh
cmake <options> ..
```

CMake options: `-DKEY=VAL`

Key | Usage | Default
--- | --- | ---
SIMD_LEVEL | SIMD processor acceleration (sse2, sse4.1, avx2, avx512f) | `avx2`
USE_FMA | Fast math | `1`
CMAKE_BUILD_TYPE | Level of optimization | `Release`
PROFILING | Tracy profiling | `0`
LOG_DEBUG | Debug logs | `0`
LOG_TRACE | Trace logs | `0`

1. Build Make
```sh
make -j<process count> <target>
```

Target | Description
--- | ---
univerxel | All in one
univerxel-server | Standalone server
univerxel-client | Light client
docs | Doxygen documentation
pages | Sphinx pages

#### Additionally

5. Recompile Vulkan shaders
```sh
cd resource/shaders-src
./compile.sh # Use glslc
GLSL="glslValidator -V" ./compile.sh # Other glslang compiler
```

6. Rebuild textures
```sh
cd resource/textures-src
./compile.sh # Build all texture dds
./merge.py <args> # Combine grey images to RGB
```


<!-- ROADMAP -->
## RoadMap

See [Features](TODO.md)

<!-- LICENSE -->
## License

Distributed under the MIT License. See [LICENSE](LICENSE) for more information.



<!-- CONTACT -->
## Contact

Shu - [/me](https://git.wadza.fr/me) - me@wadza.fr

[Public releases](https://framagit.org/univerxel/univerxel) - [Working repo](https://git.wadza.fr/me/univerxel)
