#include "iterators.hpp"

using namespace world::iterator;

std::unique_ptr<Abstract> world::iterator::Get(world::action::Shape shape, uint16_t radius) {
    switch (shape) {
    case world::action::Shape::Cube:
        return std::make_unique<Cube>(radius);
    case world::action::Shape::RawSphere:
        return std::make_unique<RawSphere>(radius);
    case world::action::Shape::SmoothSphere:
        return std::make_unique<SmoothSphere>(radius);
    default:
        return std::unique_ptr<Abstract>(nullptr);
    }
}

bool Cube::next(pair& out) {
    if (pos.z > radius)
        return false;

    out.first = pos;
    out.second = 1;

    // MAYBE: use linear idx to make branch less
    if (pos.x < radius) {
        pos.x++;
    } else {
        pos.x = -radius;
        if (pos.y < radius) {
            pos.y++;
        } else {
            pos.y = -radius;
            pos.z++;
        }
    }
    return true;
}

bool RawSphere::next(pair& out) {
    const auto delta = [](int radius, int pos) -> int {
        const auto rad = radius + .5;
        return floor(sqrt(rad * rad - pos * pos));
    };

    if (pos.z < dz) {
        pos.z++;
    } else {
        if (pos.y < dy) {
            pos.y++;
        } else {
            if (pos.x < radius) {
                pos.x++;
                dy = delta(radius, pos.x);
                pos.y = -dy;
            } else {
                return false;
            }
        }
        dz = delta(dy, pos.y);
        pos.z = -dz;
    }

    out.first = pos;
    out.second = 1;
    return true;
}

bool SmoothSphere::next(pair& out) {
    const auto delta = [](double radius, int pos) {
        return sqrt(radius * radius - pos * pos);
    };

    if (pos.z < dz) {
        const int idz = floor(dz);
        out.second = pos.z < idz ? 1 : dz - idz;
        pos.z++;
    } else {
        if (pos.y < dy) {
            pos.y++;
        } else {
            if (pos.x < radius) {
                pos.x++;
                dy = delta(radius, pos.x);
                pos.y = -ceil(dy);
            } else {
                return false;
            }
        }
        dz = delta(dy, pos.y);
        out.second = dz - floor(dz);
        pos.z = -ceil(dz);
    }

    out.first = pos;
    return true;
}